import type { LanguageInput, ThemedToken, ThemeInput } from 'shiki'
import type { TokenType } from '../../types'
import { createHighlighterCore } from 'shiki/core'
import { createJavaScriptRegexEngine } from 'shiki/engine-javascript.mjs'

async function loadLanguage(lang: SupportedLanguage): Promise<LanguageInput> {
  switch (lang) {
    case 'css':
      return (await import('shiki/dist/langs/css.mjs')).default
    case 'html':
      return (await import('shiki/dist/langs/html.mjs')).default
    case 'javascript':
      return (await import('shiki/dist/langs/javascript.mjs')).default
    case 'json':
      return (await import('shiki/dist/langs/json.mjs')).default
    case 'php':
      return (await import('shiki/dist/langs/php.mjs')).default
    case 'typescript':
      return (await import('shiki/dist/langs/typescript.mjs')).default
    case 'tsx':
      return (await import('shiki/dist/langs/tsx.mjs')).default
    default:
      throw new Error(`Unsupported language: ${lang}`)
  }
}

async function loadTheme(theme: SupportedTheme): Promise<ThemeInput> {
  switch (theme) {
    case 'github-dark':
      return (await import('shiki/dist/themes/github-dark.mjs')).default
    case 'github-light':
      return (await import('shiki/dist/themes/github-light.mjs')).default
    default:
      throw new Error(`Unsupported theme: ${theme}`)
  }
}

export type SupportedLanguage = 'css' | 'html' | 'javascript' | 'json' | 'php' | 'typescript' | 'tsx'
export type SupportedTheme = 'github-dark' | 'github-light'

interface HighlighterInstance {
  highlighter: Awaited<ReturnType<typeof createHighlighterCore>>
  loadedLanguages: Set<string>
  loadedThemes: Set<string>
}

let highlighterInstance: HighlighterInstance | null = null

interface HighlighterOptions {
  langs: SupportedLanguage[]
  themes: SupportedTheme[]
}

export async function createHighlighter({ langs, themes }: HighlighterOptions) {
  if (!langs || !themes) {
    throw new Error('Please provide both `langs` and `themes` when creating a highlighter.')
  }

  if (highlighterInstance?.highlighter) {
    for (const lang of langs) {
      if (!highlighterInstance.loadedLanguages.has(lang)) {
        const langData = await loadLanguage(lang)
        await highlighterInstance.highlighter.loadLanguage(langData)
        highlighterInstance.loadedLanguages.add(lang)
      }
    }

    for (const theme of themes) {
      if (!highlighterInstance.loadedThemes.has(theme)) {
        const themeData = await loadTheme(theme)
        await highlighterInstance.highlighter.loadTheme(themeData)
        highlighterInstance.loadedThemes.add(theme)
      }
    }

    return highlighterInstance.highlighter
  }

  try {
    const loadedLangs = await Promise.all(langs.map(loadLanguage))
    const loadedThemes = await Promise.all(themes.map(loadTheme))

    const highlighter = await createHighlighterCore({
      engine: createJavaScriptRegexEngine(),
      langs: loadedLangs,
      themes: loadedThemes,
    })

    highlighterInstance = {
      highlighter,
      loadedLanguages: new Set(langs),
      loadedThemes: new Set(themes),
    }

    return highlighter
  }
  catch (error) {
    console.error('Failed to create highlighter:', error)
    throw error
  }
}

export function processTokens(tokens: ThemedToken[]): TokenType[] {
  return tokens.map(token => ({
    content: token.content,
    color: token.color || '#000000',
    fontStyle: token.fontStyle?.toString(),
  }))
}
